// ===========================================================================
// File: W A L K A L L . C P P
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Description:
//
// This sample demonstrates the use of MSHTML as a UI-less HTML parser.
// To successfully walk the HTML Scripting Object Model that the parser
// exposes after loading the specified document, the host application must
// wait until MSHTML has finished loading the document. To track MSHTML's
// READYSTATE, the host implements a simple COM object that exposes
// IPropertyNotifySink. The host application connects to MSHTML using the
// standard connection point interfaces. As MSHTML loads the document
// it's readiness state changes. To notify the host of these changes,
// it executes IPropertyNotifySink::OnChanged passing along the DISPID
// of the property that has changed (DISPID_READYSTATE). The host uses MSHTML's
// automation interface obtained at creation time to retrieve the current value
// of this property. When the value equals READYSTATE_COMPLETE,
// MSHTML has finished loading the document.

// If the loaded document is an HTML page which contains scripts, Java Applets and/or
// ActiveX controls, and those scripts are coded for immediate execution, MSHTML will
// execute them by default. To disable this feature, the host must implement two
// additional interfaces: IOleClientSite and IDispatch. At initialization time, after
// MSHTML is instantiated, the host should query MSHTML for its IOleObject interface,
// pass MSHTML a reference to its IOleClientSite interface, query MSHTML for its IOleControl
// interface, and call OnAmbientPropertyChange(DISPID_AMBIENT_DLCONTROL). MSHTML will in
// turn query the host's IOleClientSite interface for IDispatch and then request the value
// of this property from the host. This is the hosts opportunity to control MSHTML, disabling
// the execution of scripts, etc. A full list of control flags are available in <mshtmdid.h>.

// Requirements:
//   * Internet Explorer version 4.0 or later.
//   * Microsoft Visual C++ 6.0. The code uses types (_bstr_t) introduced with VC5.
//   * Be sure that the INETSDK include and lib directories are in your INCLUDE and LIB
//     environment variables respectively.

// Instructions:
//
//  To use this sample:
//   * build it using the NMAKE command. NMAKE will create WALKALL.EXE.
//   * run WALKALL.EXE specifying the resource to download by passing an
//     URL on the command-line. use no command-line argument to default to
//     "http://www.microsoft.com".
//   * The program prints all HTML tags to the console. If an IMG tag is encountered
//     the SRC attribute is displayed.
//
// Modification History
// 30 May 1997 - MDO - created
// 12 Jun 1997 - MDO - added minimal support for IOleClientSite and IDispatch
// 22 Sep 1998 - MDO - added CApp::GetUA() to demo how to get to the window object (IHTMLWindow2) from the document (IHTMLDOcument2)
// 18 Oct 1999 - MDO - added CWinSink to sink MSHTML window object events (essentially window::onload).
// Copyright 1995-2002 Microsoft Corporation.  All Rights Reserved.
// ===========================================================================

#include "stdafx.h"



#define WM_USER_STARTWALKING	((WM_USER) + 1)

//const TCHAR c_szDefaultUrl[] = _T("http://image.baidu.com/i?z=&s=1&ct=201326592&cl=2&lm=-1&tn=baiduimage&word=%CD%F5%B7%C6&pic=1");

// Initialize the app. Load MSHTML, hook up property notification sink, etc
void PrintDISPID(DISPID dispidMember);


MyParser::MyParser()
{
	CComPtr<IOleObject> pOleObject;
	
	CComPtr<IOleControl> pOleControl;

	CComPtr<IConnectionPointContainer> pCPC;
	
	HRESULT hr;
	BOOL result = FALSE;

	m_pMSHTML    = NULL;	
	forEachImage = NULL;
	forEachLink  = NULL;			
	m_lReadyState = READYSTATE_UNINITIALIZED;
	m_dwRef = 1;
	m_event = NULL;
	m_pWinSink = NULL;
	m_pCP = NULL;
	m_dwCookie = 0;

	pUrlVector = NULL;
	userData = NULL;
	parseType = URL_Parse_By_UrlVector;
	urlTypes = URL_NONE;
	
	//hasError = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED) != S_OK; 	
	hasError = CoInitialize(NULL) != S_OK; 	
	if (!hasError)
	{
		m_event = CreateEvent(NULL, TRUE, FALSE, NULL);
		hasError = m_event == NULL;
	}

	if (SUCCEEDED(hr = CoCreateInstance( CLSID_HTMLDocument, NULL,
					CLSCTX_INPROC_SERVER, IID_IHTMLDocument2,
					(LPVOID*)&m_pMSHTML )))
	{
		//CComPtr<IHTMLWindow2> pWin;
		IHTMLWindow2* pWin = NULL;
		if (SUCCEEDED(hr = m_pMSHTML->get_parentWindow(&pWin)))
		{
			m_pWinSink = new CWinSink();
			if (m_pWinSink)
			{			
				if (SUCCEEDED(m_pWinSink->Init(pWin)))
				{
					if (SUCCEEDED(hr = m_pMSHTML->QueryInterface(IID_IOleObject, (LPVOID*)&pOleObject)))
					{
						if (SUCCEEDED(pOleObject->SetClientSite((IOleClientSite*)this)))
						{
							if (SUCCEEDED(m_pMSHTML->QueryInterface(IID_IOleControl, (LPVOID*)&pOleControl)))
							{
								hr = pOleControl->OnAmbientPropertyChange(DISPID_AMBIENT_USERMODE);

								hr = pOleControl->OnAmbientPropertyChange(DISPID_AMBIENT_DLCONTROL);	
								
								if (SUCCEEDED(hr))
								{
									// Hook up sink to catch ready state property change
									if (SUCCEEDED(hr = m_pMSHTML->QueryInterface(IID_IConnectionPointContainer, (LPVOID*)&pCPC)))
									{								
										if (SUCCEEDED(hr = pCPC->FindConnectionPoint(IID_IPropertyNotifySink, &m_pCP)))
										{									
											if (SUCCEEDED(m_pCP->Advise((LPUNKNOWN)(IPropertyNotifySink*)this, &m_dwCookie)))
											{
												result = TRUE;
											}
										}
									}
								}
								//pOleControl.Release();
							}
						}
						//pOleObject.Release();
					}					
				}
			}
		}
		//pWin.Release();
	}
/*
	if (result)
	{		
		result = m_pWinSink->setupConnection(m_pMSHTML, this);		
	}
	*/
	if (!result)
	{
		hasError = TRUE;
	}
}

MyParser::~MyParser()
{
	if (m_pWinSink)
	{
		m_pWinSink->Uninit();
		m_pWinSink->Release();
		delete m_pWinSink;
		m_pWinSink = NULL;
	}
	if (m_pCP)
	{
		if (m_dwCookie)
		{
			m_pCP->Unadvise(m_dwCookie);
			m_dwCookie = 0;
		}

		// Release the connection point
		m_pCP->Release();
		m_pCP = NULL;
	}
	if (m_pMSHTML)
	{
		m_pMSHTML->Release();
		m_pMSHTML = NULL;
	}

	if (m_event)
	{
		CloseHandle(m_event);
		m_event = NULL;
	}
	CoUninitialize();
}


STDMETHODIMP MyParser::QueryInterface(REFIID riid, LPVOID* ppv)
{
	*ppv = NULL;

	if (IID_IUnknown == riid || IID_IPropertyNotifySink == riid)
	{
		*ppv = (LPUNKNOWN)(IPropertyNotifySink*)this;
		AddRef();
		return NOERROR;
	}
	else if (IID_IOleClientSite == riid)
	{
		*ppv = (IOleClientSite*)this;
		AddRef();
		return NOERROR;
	}
	else if (IID_IDispatch == riid)
	{
		*ppv = (IDispatch*)this;
		AddRef();
		return NOERROR;
	}
	else
	{
		return E_NOTIMPL;
	}
}

STDMETHODIMP_(ULONG) MyParser::AddRef()
{
	return ++m_dwRef;
}

STDMETHODIMP_(ULONG) MyParser::Release()
{
	if (m_dwRef>0)
	{
		m_dwRef --;		
	}
	/*
	if (m_dwRef == 0)
	{
		delete this;
	}
	*/
	return m_dwRef;
}

// Fired on change of the value of a 'bindable' property
STDMETHODIMP MyParser::OnChanged(DISPID dispID)
{
	HRESULT hr;
	//TCHAR szBuff[255];

	if (DISPID_READYSTATE == dispID)
	{
		// check the value of the readystate property
		assert(m_pMSHTML);

		VARIANT vResult = {0};
		EXCEPINFO excepInfo;
		UINT uArgErr;
		DISPPARAMS dp = {NULL, NULL, 0, 0};
		if (SUCCEEDED(hr = m_pMSHTML->Invoke(DISPID_READYSTATE, IID_NULL, LOCALE_SYSTEM_DEFAULT,
			DISPATCH_PROPERTYGET, &dp, &vResult, &excepInfo, &uArgErr)))
		{
    
			#define RSLENGTH 20
			
			assert(VT_I4 == V_VT(&vResult));
			m_lReadyState = (READYSTATE)V_I4(&vResult);
			switch (m_lReadyState)
			{	
  			case READYSTATE_UNINITIALIZED:	//= 0,
          // CODE REVIEW: Replace banned lstrcpy API.
          // lstrcpy(szReadyState, _T("Uninitialized"));
				//StringCchCopy(szReadyState, RSLENGTH, _T("Uninitialized"));
  				break;
  			case READYSTATE_LOADING: //	= 1,
          // CODE REVIEW: Replace banned lstrcpy API.
          // lstrcpy(szReadyState, _T("Loading"));
				//StringCchCopy(szReadyState, RSLENGTH, _T("Loading"));
  				break;	
  			case READYSTATE_LOADED:	// = 2,
          // CODE REVIEW: Replace banned lstrcpy API.
          // lstrcpy(szReadyState, _T("Loaded"));
				//StringCchCopy(szReadyState, RSLENGTH, _T("Loaded"));
  				break;
  			case READYSTATE_INTERACTIVE: //	= 3,
          // CODE REVIEW: Replace banned lstrcpy API.
          // lstrcpy(szReadyState, _T("Interactive"));
				//StringCchCopy(szReadyState, RSLENGTH, _T("Interactive"));
  				break;
  			case READYSTATE_COMPLETE: // = 4
          // CODE REVIEW: Replace banned lstrcpy API.
  				// lstrcpy(szReadyState, _T("Complete"));
				//StringCchCopy(szReadyState, RSLENGTH, _T("Complete"));
				
  				BOOL fRet = PostThreadMessage(GetCurrentThreadId(),
  									WM_USER_STARTWALKING,
  									(WPARAM)0,
  									(LPARAM)0);
				

				//SetEvent(m_event);
  				break;
			}

      // CODE REVIEW: Replace Banned API.
			// wsprintf(szBuff, _T("OnChanged: readyState = %s\n"), szReadyState);
			//StringCchPrintf(szBuff, 255, _T("OnChanged: readyState = %s\n"), szReadyState);
			VariantClear(&vResult);
		}
		else
		{
      // CODE REVIEW: Replace banned lstrcpy API.
      // lstrcpy(szBuff, _T("Unable to obtain readyState value\n"));
			//StringCchCopy(szBuff, 255, _T("Unable to obtain readyState value\n"));
		}
	}
	else
	{
    // CODE REVIEW: Replace Banned API.
		// wsprintf(szBuff, _T("OnChanged: %d\n"), dispID);
		//StringCchPrintf(szBuff, 255, _T("OnChanged: %d\n"), dispID);
	}

	//ODS(szBuff);
	//cout << szBuff;

	return NOERROR;
}

STDMETHODIMP MyParser::OnRequestEdit(DISPID dispID)
{
	// Property changes are OK any time as far as this app is concerned
	//TCHAR szBuff[255];
  // CODE REVIEW: Replace Banned API.
	// wsprintf(szBuff, _T("OnRequestEdit: %d\n"), dispID);
	//StringCchPrintf(szBuff, 255, _T("OnRequestEdit: %d\n"), dispID);
	//ODS(szBuff);

	return NOERROR;
}

// Return the protocol associated with the specified URL
INTERNET_SCHEME MyParser::GetScheme(LPCTSTR szURL)
{
	URL_COMPONENTS urlComponents;
	DWORD dwFlags = 0;
	INTERNET_SCHEME nScheme = INTERNET_SCHEME_UNKNOWN;

  // CODE REVIEW: Dangerous API. Ensure zeroed buffer size in bytes. It is.
	ZeroMemory((PVOID)&urlComponents, sizeof(URL_COMPONENTS));
	urlComponents.dwStructSize = sizeof(URL_COMPONENTS);

	if (szURL)
	{
		if (InternetCrackUrl(szURL, 0, dwFlags, &urlComponents))
		{
			nScheme = urlComponents.nScheme;
		}
	}
	return nScheme;
}

void MyParser::handleUrl(DWORD urlType, UrlStruct& urlStruct)
{
	switch (parseType)
	{
	case URL_Parse_By_UrlVector:
		if (pUrlVector)
		{
			pUrlVector->push_back(urlStruct);
		}
		break;
	case URL_Parse_By_Callback:
		if (urlCallback)
		{
			urlCallback(urlType, urlStruct.url, userData);
		}
		break;
	}
}


HRESULT MyParser::WalkAll()
{
	HRESULT hr = E_UNEXPECTED;
	if (urlTypes & URL_IMAGES)
	{
		if (!SUCCEEDED(hr = doWalkAll(URL_IMAGES))) 
		{
			return hr;
		}
	}
	if (urlTypes & URL_LINKS)
	{
		if (!SUCCEEDED(hr = doWalkAll(URL_LINKS))) 
		{
			return hr;
		}
	}
	if (urlTypes & URL_ANCHORS)
	{
		if (!SUCCEEDED(hr = doWalkAll(URL_ANCHORS))) 
		{
			return hr;
		}
	}
	if (urlTypes & URL_TAGS)
	{
		if (!SUCCEEDED(hr = doWalkAll(URL_TAGS))) 
		{
			return hr;
		}
	}
	return hr;
}

HRESULT MyParser::doWalkAll(DWORD urlType)
{
	HRESULT hr;
	IHTMLElementCollection* pColl = NULL;

	IHTMLImgElement* pImage = NULL;
	IHTMLLinkElement* pLink = NULL;	
	IHTMLAnchorElement* pAnchor = NULL;

	BSTR bstr;

	UrlStruct urlStruct;

	//assert(m_pMSHTML);
	if (!m_pMSHTML)
	{
		return E_UNEXPECTED;
	}

	if (READYSTATE_COMPLETE != m_lReadyState)
	{		
		//DebugBreak();
		return E_UNEXPECTED;
	}

	// retrieve a reference to the ALL collection
	switch (urlType)
	{
	case URL_IMAGES:
		hr = m_pMSHTML->get_images(&pColl);
		break;
	case URL_LINKS:
		hr = m_pMSHTML->get_links(&pColl);
		break;
	case URL_ANCHORS:
		hr = m_pMSHTML->get_anchors(&pColl);
		break;
	case URL_TAGS:
		hr = m_pMSHTML->get_all(&pColl);
		break;
	}

	if (SUCCEEDED(hr))
	{
		long cElems;
		// retrieve the count of elements in the collection
		if (SUCCEEDED(hr = pColl->get_length( &cElems )))
		{
			// for each element retrieve properties such as TAGNAME and HREF
			for ( int i=0; i<cElems; i++ )
			{
				VARIANT vIndex;
				vIndex.vt = VT_UINT;
				vIndex.lVal = i;
				VARIANT var2 = { 0 };
				LPDISPATCH pDisp;
				BSTR bstrTag, bstrName;

				if (SUCCEEDED(hr = pColl->item( vIndex, var2, &pDisp )))
				{
					IHTMLElement* pElem = NULL;
					if (SUCCEEDED(hr = pDisp->QueryInterface(IID_IHTMLElement, (LPVOID*)&pElem )))
					{
						// if the element is an <IMG> get its SRC attribute
						switch (urlType)
						{
						case URL_IMAGES:						
							if (SUCCEEDED(hr = pElem->QueryInterface(IID_IHTMLImgElement, (LPVOID*)&pImage )))
							{
								pImage->get_src(&bstr);
								if (bstr)
								{
									wcscpy(urlStruct.url, bstr);
									handleUrl(urlType, urlStruct);
									//urlVector.push_back(urlStruct);
									SysFreeString(bstr);
								}
								pImage->Release();
							}
							break;
						case URL_LINKS:
									
							if (SUCCEEDED(hr=pElem->QueryInterface(IID_IHTMLLinkElement, (LPVOID*)&pLink)))
							{								
								pLink->get_href(&bstr);
								if (bstr)
								{
									wcscpy(urlStruct.url, bstr);
									//urlVector.push_back(urlStruct);
									handleUrl(urlType, urlStruct);
									SysFreeString(bstr);
								}
								pLink->Release();
							}
							break;
						case URL_ANCHORS:							
							if (SUCCEEDED(hr=pElem->QueryInterface(IID_IHTMLAnchorElement, (LPVOID*)&pAnchor)))
							{
								assert(pAnchor);
								pAnchor->get_href(&bstr);
								if (bstr)
								{
									wcscpy(urlStruct.url, bstr);
									//urlVector.push_back(urlStruct);
									handleUrl(urlType, urlStruct);
									SysFreeString(bstr);
								}
								pAnchor->Release();
							}
							break;
						case URL_TAGS:
							if (SUCCEEDED(hr=pElem->get_tagName(&bstrTag)))
							{
								if (_wcsicmp(bstrTag, L"a")==0)
								{
									if (SUCCEEDED(hr=pElem->toString(&bstrName)))
									{
										wcscpy(urlStruct.url, bstrName);
										//urlVector.push_back(urlStruct);
										handleUrl(urlType, urlStruct);
										SysFreeString(bstrName);
									}
								}
								SysFreeString(bstrTag);
							}
							break;
						}
						pElem->Release();
					}					
					pDisp->Release();
				}				
			}
		}
		pColl->Release();
	}
	PostQuitMessage(0);
	return hr;
}


BOOL MyParser::parseUrl(LPCWSTR url, DWORD myUrlType, PUrlVector myPUrlVector)
{
	pUrlVector = myPUrlVector;
	urlTypes = myUrlType;
	parseType = URL_Parse_By_UrlVector;
	return doParseUrl(url);
}


BOOL MyParser::parseUrl(LPCWSTR url, DWORD myUrlType, UrlCallbackType myUrlCallback, LPVOID myUserData)
{
	urlCallback = myUrlCallback;
	userData = myUserData;
	urlTypes = myUrlType;
	parseType = URL_Parse_By_Callback;
	return doParseUrl(url);
}

BOOL MyParser::parseUrl2(LPCWSTR url, DWORD myUrlType, UrlCallbackType myUrlCallback, LPVOID myUserData)
{
	urlCallback = myUrlCallback;
	userData = myUserData;
	urlTypes = myUrlType;
	parseType = URL_Parse_By_Callback;
	return doParseUrl2(url);
}

BOOL MyParser::doParseUrl(LPCTSTR url)
{
	MSG msg;
	if (SUCCEEDED(LoadURLFromMoniker(url)))
	{
		while (GetMessage(&msg, NULL, 0, 0))
		{
			if (WM_USER_STARTWALKING == msg.message && NULL == msg.hwnd)
			{
				WalkAll();
			}
			else
			{
				DispatchMessage(&msg);
			}
		}
		return TRUE;
	}
	return FALSE;
}

BOOL MyParser::doParseUrl2(LPCTSTR lpFileName)
{
	MSG msg;
	LPBYTE lpBuffer = NULL;
	DWORD dwSize = 0;
	BOOL result = FALSE;
	HGLOBAL hMem = NULL;
	CComPtr<IStream> pStream = NULL;
	LPVOID pMem = NULL;

	if (LoadURLFromFile(lpFileName, lpBuffer, dwSize))
	{		
		if ((hMem = GlobalAlloc(GMEM_MOVEABLE|GMEM_NODISCARD, dwSize)) != NULL)
		{
			if ((pMem = GlobalLock(hMem)) != NULL)
			{
				memcpy(pMem, lpBuffer, dwSize);
				free(lpBuffer);
				if (SUCCEEDED(CreateStreamOnHGlobal(hMem, TRUE, &pStream)))
				{
					CComQIPtr<IPersistStreamInit> spPersistStream(m_pMSHTML);
					if (spPersistStream)
					{
						spPersistStream->InitNew();
						spPersistStream->Load(pStream);
						result = TRUE;
	
						while (GetMessage(&msg, NULL, 0, 0))
						{
							if (WM_USER_STARTWALKING == msg.message && NULL == msg.hwnd)
							{
								WalkAll();
							}
							else
							{
								DispatchMessage(&msg);
							}
						}
					}
				}
				GlobalUnlock(hMem);
			}
			GlobalFree(hMem);
		}		
	}
	return result;
}



// Use an asynchronous Moniker to load the specified resource
HRESULT MyParser::LoadURLFromMoniker(LPCTSTR url)
{
	HRESULT hr;

	LPMONIKER pMk = NULL;
	LPBINDCTX pBCtx = NULL;
	LPPERSISTMONIKER pPMk = NULL;

	//if (FAILED(hr = CreateURLMoniker(NULL, wszURL, &pMk)))
	if (FAILED(hr = CreateURLMoniker(NULL, url, &pMk)))
	{
		return hr;
	}

	if (FAILED(hr = CreateBindCtx(0, &pBCtx)))
	{
		goto Error;
	}

	// Use MSHTML moniker services to load the specified document
	if (SUCCEEDED(hr = m_pMSHTML->QueryInterface(IID_IPersistMoniker, (LPVOID*)&pPMk)))
	{
		hr = pPMk->Load(FALSE, pMk, pBCtx, STGM_READ);
	}
		
Error:
	if (pMk) pMk->Release();
	if (pBCtx) pBCtx->Release();
	if (pPMk) pPMk->Release();
	return hr;
}

// A more traditional form of persistence.
// MSHTML performs this asynchronously as well.
bool MyParser::LoadURLFromFile(LPCTSTR lpFileName, LPBYTE& lpBuffer, DWORD& dwSize)
{
	DWORD bytesRead = 0;
	HANDLE hFile = INVALID_HANDLE_VALUE;
	//LPBYTE lpBuffer = NULL;
	if ((hFile = CreateFile(lpFileName, GENERIC_READ, FILE_SHARE_DELETE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL))!= INVALID_HANDLE_VALUE)
	{
		if ((dwSize = GetFileSize(hFile, NULL))!= INVALID_FILE_SIZE)
		{
			if ((lpBuffer = (LPBYTE)malloc(dwSize))!= NULL)
			{
				if (ReadFile(hFile, lpBuffer, dwSize, &bytesRead, NULL))
				{
					return true;
				}
			}
		}
		CloseHandle(hFile);
	}
	return false;
}



#if 0
	IPersistFile* pPF = NULL;
	if ( SUCCEEDED(hr = m_pMSHTML->QueryInterface(IID_IPersistFile, (LPVOID*) &pPF)))
	{		
		hr = pPF->Load(fileName, STGM_READ);
		pPF->Release();
	}
//#else
	IPersistStream* pPs = NULL;
	if ( SUCCEEDED(hr = m_pMSHTML->QueryInterface(IID_IPersistStream, (LPVOID*) &pPs)))
	{		
		hr = pPs->Load(fileName);
		pPs->Release();
	}


	return hr;	
}
#endif


// MSHTML Queries for the IDispatch interface of the host through the IOleClientSite
// interface that MSHTML is passed through its implementation of IOleObject::SetClientSite()
STDMETHODIMP MyParser::Invoke(DISPID dispIdMember,
            REFIID riid,
            LCID lcid,
            WORD wFlags,
            DISPPARAMS __RPC_FAR *pDispParams,
            VARIANT __RPC_FAR *pVarResult,
            EXCEPINFO __RPC_FAR *pExcepInfo,
            UINT __RPC_FAR *puArgErr)
{
	if (!pVarResult)
	{
		return E_POINTER;
	}

	PrintDISPID(dispIdMember);

	switch(dispIdMember)
	{
	case DISPID_AMBIENT_DLCONTROL:
		// respond to this ambient to indicate that we only want to
		// download the page, but we don't want to run scripts,
		// Java applets, or ActiveX controls
		V_VT(pVarResult) = VT_I4;
		V_I4(pVarResult) =  DLCTL_DOWNLOADONLY |
							DLCTL_NO_SCRIPTS |
							DLCTL_NO_JAVA |
							DLCTL_NO_DLACTIVEXCTLS |
							DLCTL_NO_RUNACTIVEXCTLS;
		break;
	case DISPID_AMBIENT_USERMODE:
		// put MSHTML into design mode
		V_VT(pVarResult) = VT_BOOL;
		V_BOOL(pVarResult) = VARIANT_FALSE;
		break;
	default:
		return DISP_E_MEMBERNOTFOUND;
	}

	return NOERROR;
}

// Diagnostic helper to discover what ambient properties MSHTML
// asks of the host
void PrintDISPID(DISPID dispidMember)
{
	#define ALEN(x) (sizeof(x)/(sizeof(x[0])))

	typedef struct {
		DISPID dispid;
		LPCTSTR szDesc;
	} DISPIDDESC;

	static DISPIDDESC aDISPIDS[] = {	
							{DISPID_AMBIENT_DLCONTROL, _T("DISPID_AMBIENT_DLCONTROL")}, // (-5512)
							{DISPID_AMBIENT_USERAGENT, _T("DISPID_AMBIENT_USERAGENT")}, // (-5513)
							{DISPID_AMBIENT_BACKCOLOR, _T("DISPID_AMBIENT_BACKCOLOR")},  //        (-701)
							{DISPID_AMBIENT_DISPLAYNAME, _T("DISPID_AMBIENT_DISPLAYNAME")},  //      (-702)
							{DISPID_AMBIENT_FONT, _T("DISPID_AMBIENT_FONT")},  //             (-703)
							{DISPID_AMBIENT_FORECOLOR, _T("DISPID_AMBIENT_FORECOLOR")},  //        (-704)
							{DISPID_AMBIENT_LOCALEID, _T("DISPID_AMBIENT_LOCALEID")},  //         (-705)
							{DISPID_AMBIENT_MESSAGEREFLECT, _T("DISPID_AMBIENT_MESSAGEREFLECT")},  //   (-706)
							{DISPID_AMBIENT_SCALEUNITS, _T("DISPID_AMBIENT_SCALEUNITS")},  //       (-707)
							{DISPID_AMBIENT_TEXTALIGN, _T("DISPID_AMBIENT_TEXTALIGN")},  //        (-708)
							{DISPID_AMBIENT_USERMODE, _T("DISPID_AMBIENT_USERMODE")},  //         (-709)
							{DISPID_AMBIENT_UIDEAD, _T("DISPID_AMBIENT_UIDEAD")},  //           (-710)
							{DISPID_AMBIENT_SHOWGRABHANDLES, _T("DISPID_AMBIENT_SHOWGRABHANDLES")},  //  (-711)
							{DISPID_AMBIENT_SHOWHATCHING, _T("DISPID_AMBIENT_SHOWHATCHING")},  //     (-712)
							{DISPID_AMBIENT_DISPLAYASDEFAULT, _T("DISPID_AMBIENT_DISPLAYASDEFAULT")},  // (-713)
							{DISPID_AMBIENT_SUPPORTSMNEMONICS, _T("DISPID_AMBIENT_SUPPORTSMNEMONICS")},  // (-714)
							{DISPID_AMBIENT_AUTOCLIP, _T("DISPID_AMBIENT_AUTOCLIP")},  //         (-715)
							{DISPID_AMBIENT_APPEARANCE, _T("DISPID_AMBIENT_APPEARANCE")},  //       (-716)
							{DISPID_AMBIENT_PALETTE, _T("DISPID_AMBIENT_PALETTE")},  //          (-726)
							{DISPID_AMBIENT_TRANSFERPRIORITY, _T("DISPID_AMBIENT_TRANSFERPRIORITY")},  // (-728)
							{DISPID_AMBIENT_RIGHTTOLEFT, _T("DISPID_AMBIENT_RIGHTTOLEFT")},  //      (-732)
							{DISPID_AMBIENT_TOPTOBOTTOM, _T("DISPID_AMBIENT_TOPTOBOTTOM")}  //      (-733)
	};

	//TCHAR szBuff[255];

	for (int i = 0; i < ALEN(aDISPIDS); i++)
	{
		if (dispidMember == aDISPIDS[i].dispid)
		{			
			return;
		}
	}
}

// Given a document reference (IHTMLDocument2), get the user-agent string
// document.parentWindow.navigator.userAgent
HRESULT MyParser::GetUA(BSTR *pbstrUA)
{
	HRESULT hr;
	IHTMLWindow2* pWindow = NULL;

	assert(m_pMSHTML);
	if (!m_pMSHTML)
	{
		return E_UNEXPECTED;
	}

	// m_pMSHTML caches an IHTMLDocument2 interface pointer
	hr = m_pMSHTML->get_parentWindow(&pWindow);
	if (FAILED(hr) || !pWindow)
	{
		return hr;
	}

	IOmNavigator* pNavigator = NULL;
	// get_clientInformation == get_navigator
	hr = pWindow->get_clientInformation(&pNavigator);
	if (FAILED(hr) || !pNavigator)
	{
		goto Error;
	}

	hr = pNavigator->get_userAgent(pbstrUA);


Error:
	if (pWindow)
	{
		pWindow->Release();
	}

	if (pNavigator)
	{
		pNavigator->Release();
	}

	return hr;
}

void PrintBSTR(BSTR bstr)
{
#ifdef _DEBUG
	_bstr_t _bstr;
	_bstr = bstr;
	PrintBSTR(_bstr);
#endif
}

void PrintBSTR(_bstr_t _bstr)
{

}
