// ===========================================================================
// File: W I N S I N K . C P P
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//
//  - CWinSink implements a simple object that traps MSHTML window events such as window::onload.
//    DHTML authors simply do the following in an HTML page:
//        function window_load()
//        {
//           alert("window::onload fired");
//        }
//
//        window.onload = window_load;
//
//    C++ developers do the following:
//        Instantiate a host object that implements IDispatch
//        QI an interface exposed by MSHTML for IConnectionPointContainer (ICPC)
//        Request a specific connection point via ICPC
//        Call Advise on the connect point, passing a reference to the host object
//
//        MSHTML will call the host's IDispatch::Invoke implementation when an event occurs.
//
//  - For a complete list of window events see dispinterface HTMLWindowEvents2 in mshtml.idl.
//
// Copyright 1995-2002 Microsoft Corporation.  All Rights Reserved.

#include "stdafx.h"


HRESULT CWinSink::setupConnection(IHTMLDocument2* pDoc2, MyParser* pApp)
{
	HRESULT result = E_FAIL;
	
	CComPtr<IConnectionPointContainer> pCPC;
	if (SUCCEEDED(pDoc2->QueryInterface(IID_IConnectionPointContainer, (LPVOID*)&pCPC)))
	{
		if (SUCCEEDED(pCPC->FindConnectionPoint(IID_IPropertyNotifySink, &m_pCP)))
		{
			if (SUCCEEDED(m_pCP->Advise((LPUNKNOWN)(IPropertyNotifySink*)pApp, &m_dwCookie)))
			{
				result = NOERROR;
			}
		}			
	}
	return result;
}


HRESULT CWinSink::Uninit()
{
	HRESULT hr = NOERROR;

	if (m_pCP)
	{
		if (m_dwCookie)
		{
			hr = m_pCP->Unadvise(m_dwCookie);
			m_dwCookie = 0;
		}
		 m_pCP->Release();
		 m_pCP = NULL;
	}
	if (m_pWin)
	{
		m_pWin->Release();
		m_pWin = NULL;
	}

	return NOERROR;
}

HRESULT CWinSink::Init(IHTMLWindow2* pWin)
{
	HRESULT hr = E_FAIL;
	CComPtr<IConnectionPointContainer> pCPC;

	if (m_pWin)
	{
		m_pWin->Release();
		m_pWin = NULL;
	}
	m_pWin = pWin;

	if (SUCCEEDED(hr = pWin->QueryInterface(IID_IConnectionPointContainer, (LPVOID*)&pCPC)))
	{
		if (SUCCEEDED(hr = pCPC->FindConnectionPoint(DIID_HTMLWindowEvents2, &m_pCP)))
		{
			m_pCP->Advise((LPUNKNOWN)this, &m_dwCookie);
			hr = NOERROR;
		}
	}
	return hr;
}

STDMETHODIMP CWinSink::QueryInterface(REFIID riid, LPVOID* ppv)
{
	*ppv = NULL;
	if (IID_IUnknown == riid)
	{
		*ppv = (LPUNKNOWN)this;
		AddRef();
		return NOERROR;
	}
	else 
	{
		if (IID_IDispatch == riid)
		{
			*ppv = (IDispatch*)this;
			AddRef();
			return NOERROR;
		}
		else
		{
	
			return E_NOTIMPL;
		}
	}
}


/*
	*ppv = NULL;
	HRESULT hr = E_NOTIMPL;

	switch (riid)
	{
	case IID_IUnknown:	
		*ppv = (LPUNKNOWN)this;
		AddRef();
		hr =  NOERROR;
		break;
	case IID_IDispatch:
		*ppv = (IDispatch*)this;
		AddRef();
		hr = NOERROR;
		break;
	default:
		break;
	}
	return hr;
	*/


STDMETHODIMP_(ULONG) CWinSink::AddRef()
{
	return ++m_dwRef;
}

STDMETHODIMP_(ULONG) CWinSink::Release()
{ 
	if (m_dwRef > 0)
	{	
		m_dwRef --;		
	}
	// qh : don't delete
/*
	if (m_dwRef == 0)
	{
		delete this;
	}
	*/
	return m_dwRef;
}

// OA events are fired through the IDispatch::Invoke of the sink object
STDMETHODIMP CWinSink::Invoke(DISPID dispIdMember,
            REFIID riid,
            LCID lcid,
            WORD wFlags,
            DISPPARAMS __RPC_FAR *pDispParams,
            VARIANT __RPC_FAR *pVarResult,
            EXCEPINFO __RPC_FAR *pExcepInfo,
            UINT __RPC_FAR *puArgErr)
{
	HRESULT hr = E_FAIL;
	if (!pVarResult)
	{
		return E_POINTER;
	}

	switch(dispIdMember)
	{
	case DISPID_HTMLWINDOWEVENTS2_ONLOAD:
		hr = NOERROR;
		break;
	default:
		hr = DISP_E_MEMBERNOTFOUND;
		break;
	}
	return hr;
}